using System;
using System.Web;
using System.IO ;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Globalization;


namespace ImageResizer
{
	public class ImageResizeHandler: IHttpHandler 
	{            

		int Width = 0;
		int Height = 0;  


		public void ProcessRequest(System.Web.HttpContext context)
		{
			string FilePath = null;
			byte[] buffer;




			if(context.Request.Params["height"] != null)
			{
				try
				{
					Height = int.Parse(context.Request.Params["height"]);
				}
				catch
				{
					Height = 0;
				}
			}  

			if(context.Request.Params["width"] != null)
			{
				try
				{
					Width = int.Parse(context.Request.Params["width"]);
				}
				catch
				{
					Width = 0;
				}
			}                  

			FilePath = RemoveImgSuffix(context.Request.PhysicalPath);
			if (Width <= 0 && Height <=0)
			{                        

				context.Response.Clear();
				context.Response.ContentType = getContentType(FilePath );
				context.Response.WriteFile(FilePath);
				context.Response.End();  

			}
			else 
			{
				// check if we already have it in the cache 
				buffer = (byte[]) context.Cache[context.Request.RawUrl] ; 

				context.Response.Clear();
	 		        string contentType = getContentType(FilePath);
                		// not a Image type : try to get an Icon
		                if (contentType == "")
                		{
	        	            contentType = @"Image/png";
				    if (null == buffer)	
				    {
	        	        	    buffer = ExtractAssociatedIconEx(FilePath, Width, Height);
		   			    context.Cache.Insert(context.Request.RawUrl,buffer);
				    }
		                }
                		else
		                {
				    if (null == buffer)	
				    {
	                		    buffer = getResizedImage(FilePath, Width, Height);
		   			    context.Cache.Insert(context.Request.RawUrl,buffer);
				    }	

		                }
                		context.Response.ContentType = contentType ; 

		       	/*  Not implemented yet: Add Support for IfModifiedSince 
			        FileInfo fi1 = new FileInfo(FilePath);
                		DateTime dw = fi1.LastWriteTimeUtc;
		                DateTimeFormatInfo myDTFI = new CultureInfo("en-US", false).DateTimeFormat;

		                string lastMod = dw.ToString("r");
                		context.Response.AppendHeader("Last-Modified", lastMod);
			*/
		   		context.Response.OutputStream.Write(buffer, 0, buffer.Length);
				context.Response.End();
			}
		}

		private string RemoveImgSuffix(string path)
		{
			return path.Substring(0,path.Length-4);
		}

		public bool IsReusable
		{
			get
			{
				return false;
			}
		}

        byte[] ExtractAssociatedIconEx(String path,int width,int height )
        {
            Icon ico = Icon.ExtractAssociatedIcon(path);
            Bitmap bmpIco = ico.ToBitmap();
            System.IO.MemoryStream outStream = new System.IO.MemoryStream();
            Bitmap imgOut = ResizedImage(bmpIco, width, height);
            imgOut.Save(outStream, ImageFormat.Png);
            return outStream.ToArray();
        }

	byte[] getResizedImage(String path ,int width,int height)
	{
            System.IO.MemoryStream outStream = new System.IO.MemoryStream();
            Bitmap imgIn = new Bitmap(path);
            Bitmap imgOut = ResizedImage(imgIn, width, height);
            imgOut.Save(outStream, getImageFormat(path));
            return outStream.ToArray();
		}

        Bitmap ResizedImage(Bitmap imgIn, int width, int height)
        {
            double y = imgIn.Height;
            double x = imgIn.Width;

            double factor = 1;
            if (width > 0)
            {
                factor = width / x;
            }
            else if (height > 0)
            {
                factor = height / y;
            }

            Bitmap imgOut = new Bitmap((int)(x * factor), (int)(y * factor));
            Graphics g = Graphics.FromImage(imgOut);
            g.Clear(Color.White);

            g.CompositingMode = CompositingMode.SourceCopy;
            g.CompositingQuality = CompositingQuality.HighQuality;
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
            g.SmoothingMode = SmoothingMode.AntiAlias;

            g.DrawImage(imgIn, new Rectangle(0, 0, (int)(factor * x),
                (int)(factor * y)),
                new Rectangle(0, 0, (int)x, (int)y), GraphicsUnit.Pixel);
            return imgOut; 
        }  

		string getContentType(String path)
		{
			switch (Path.GetExtension(path).ToLower()) 
			{
				case ".bmp": return "Image/bmp";
				case ".gif": return "Image/gif";
				case ".jpg": return "Image/jpeg";
				case ".png": return "Image/png";
                default: break; 
                    
                    
                    
			}
			return "";
		}
 
		ImageFormat getImageFormat(String path)
		{
			switch (Path.GetExtension(path)) 
			{
				case ".bmp": return ImageFormat.Bmp;
				case ".gif": return ImageFormat.Gif;
				case ".jpg": return ImageFormat.Jpeg;
				case ".png": return ImageFormat.Png;
                default: return ImageFormat.Png; 
			}
			return ImageFormat.Jpeg;
		}
	}
}
